// Using associative tables is the recommended way of representing many-to-many relationships.
//https://en.wikipedia.org/wiki/Many-to-many_%28data_model%29
//http://web.csulb.edu/colleges/coe/cecs/dbdesign/dbdesign.php?page=manymany.php

let rightsEnum = [
    'admin',
    'speaker',
    'member'
];

exports.up = function (knex, Promise) {
    return knex.schema.createTable('metagroup_memberships', function (table) {
        table.timestamps(true, true);
        table.string('simple_group_gid',128).notNullable()
            .references('gid').inTable('groups' /*'simple_groups'*/)
            .onDelete('CASCADE'); //delete row from group_memberships if member is deleted
        table.string('meta_group_gid',128).notNullable()
            .references('gid').inTable('groups' /*'meta_groups'*/)
            .onDelete('CASCADE');
        /*
        ^ as stated in https://www.postgresql.org/docs/10/static/ddl-inherit.html#DDL-INHERIT-CAVEATS,
        "A serious limitation of the inheritance feature is that indexes (including unique constraints) and foreign key constraints only apply to single tables, not to their inheritance children."
        it is not possible (with PostgresQL v10) to use both inheritance and foreign keys at once.
        until it is, or a workaround is found, we simply don't maintain child-level referential integrity
        and we hope all the rest will work, e.g. simple_group_gids will indeed refer to simple groups, and idem for meta_group_gids.
        TODO: maybe, check if PostgresQL's latest version enables this feature?
        */
        table.enum('rights', rightsEnum).notNullable(); //TODO: this is not used by GraphQL schema!
    });
};

exports.down = function (knex, Promise) {
    return knex.schema.dropTable('metagroup_memberships');
};
