Mémo d'introduction à Knex.js pour les nuls
===

```info
Note : ce mémo a été rédigé à l'origine pour le CONTRIBUTING.md de shitpost-backend, donc fait parfois référence à des fichiers de ce projet. C'est indiqué à chaque fois lorsque c'est le cas.
```

J'utilise Knex.js avec PostgreSQL (comme dans sigma).

## knexfile.js et knex_init.js

`knexfile.js`:
- Fichier utilisé par les command-line-tools de knex (`knex migrate:*` et `knex seed:*`)
- Le rôle principal du fichier est de préciser (quelle db on utilise, et) où stocker les fichiers de migrations Knex ainsi que les _seeds_.
- Malheureusement, la documentation de knex est très pauvre sur quoi mettre dans knexfile.js... [2018-10-13]
    - https://github.com/tgriesser/knex/issues/563
    - https://github.com/tgriesser/knex/issues/1223
- Destiné aux migrations et aux seeds, pas aux fichiers js. 
    - Correspond à cette partie de la doc : https://knexjs.org/#knexfile (Migrations/CLI/knexfile)
- Pour utiliser les commandes `knex` en CLI, il faut se placer dans le directory qui contient ce fichier.

`knex_init.js`: 
- Configure et exporte un objet knex permettant les requêtes SQL
- Spécifie la bdd à laquelle se connecter, son adresse, en tant que quel user (en l'occurrence quel Role) se connecter au serveur, et son mdp.
- Contrairement à `knexfile.js`, il est destiné au serveur Nodejs (i.e. aux fichiers js). 
    - Correspond à cette partie : https://knexjs.org/#Installation-client ("Initializing the library")

Il est évident que ces deux fichiers ont en gros le même contenu ! C'est un "problème" [connu](https://github.com/tgriesser/knex/issues/1223), et actuellement (2018-10, knexjs v0.15.2) c'est bien comme ça qu'on est censé faire. (Mais en fait ce n'est pas un gros problème, juste un peu confusing, et engendre un peu de redondance.)

## "Migrations" et "Seeds"
https://gist.github.com/NigelEarle/70db130cc040cc2868555b29a0278261

### Migrations
> Migrations are a way to make database changes or updates, like creating or dropping tables, as well as updating a table with new columns with constraints, *via generated scripts*. We can build these scripts via the command line using `knex` command line tool.

En gros, les migrations permettent de définir, de façon "statique" et donc plus lisible, le schéma de la BDD. 
Autre cas de figure : elles permettent de définir de façon statique les modifications à apporter au schéma d'une BDD existante. C'est moins utile pour nous, mais sert à apporter des modifications à la BDD d'une application déjà en production, en s'assurant que si ça casse quelque chose on aura la possibilité de revenir en arrière.

Elles consistent en une liste de "commandes knex" qui disent exactement quoi faire. C'est le même principe que quand on exporte une BDD SQL : le fichier obtenu est en fait une liste de commandes SQL, qui lorsqu'executée, recrée la BDD exportée. 
Sauf qu'en plus, il y a aussi une liste de "commandes knex" qui disent quoi faire pour *défaire la migration*, c'est le `exports.down` (alors que la liste disant quoi faire pour faire la migration est dans `exports.up`).

### Seeds
> Similar to migrations, the `knex` module allows us to *create scripts, called seed files, to insert initial data into our tables*!

Les seeds permettent d'insérer des données dans la BDD. Bien sûr, il est beaucoup plus logique de définir (`knex migrate:make create_blablabla`) et d'exécuter (`knex migrate:latest`) ses migrations, avant de gérer les seeds.

### Choix du schéma et rédaction des migrations
**[spécifique à shitpost :]**
On a défini les migrations (`./db/migrations/*`) en se basant sur le schéma GraphQL (`typeDefs.js`) (et pas l'inverse), ce qui se voit bien puisqu'on suit presque exactement ce dernier. 

**[spécifique à shitpost :]**
Un point particulier cependant : comme expliqué en commentaires du migration-file `./db/migrations/[timestamp]_create_comments.js`, on n'a pas déclaré le champ "forMessage" de la table "comments" comme ayant "messages.id" comme foreign key, car ce n'est pas possible avec PostgreSQL v10.

### Un petit problème avec les seeds
Puisqu'à chaque `knex seed:run` on DELETE toutes les entrées de toutes les tables avant de les reconstruire (cf. ligne 3 de tous les seedfiles : "`// Deletes ALL existing entries. return knex('table_name').del().then(...)`"), on tombe sur ce genre de problème :
```
error: update or delete on table "channels" violates foreign key constraint "messages_channel_foreign" on table "messages"
```
Solution : spécifier dans le schéma (fichiers migrations) ce qu'il faut faire quand l'objet référencé par une colonne FOREIGN KEY est détruit, avec l'option ON DELETE (reflété par `.onDelete(...)` de knex). https://www.postgresql.org/docs/current/static/ddl-constraints.html#DDL-CONSTRAINTS-FK
